#
#  engine
#
#  Playout Daemon for autoradio project
#
#
#  Copyright (C) 2017-2018 Gottfried Gaisbauer <gottfried.gaisbauer@servus.at>
#
#  This file is part of engine.
#
#  engine is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  any later version.
#
#  engine is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with engine. If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys
import threading
import json
import queue
import traceback
import urllib
import logging

from mutagen.flac import FLAC
from datetime import datetime, timedelta

from libraries.database.broadcasts import Schedule, Playlist, PlaylistEntry, PlaylistEntryMetaData
from modules.communication.redis.messenger import RedisMessenger
from modules.scheduling.calender_fetcher import CalendarFetcher


class AuraCalendarService(threading.Thread):
    """ 
    The `AuraCalendarService` retrieves all current schedules and related
    playlists including audio files from the configured API endpoints and
    stores it in the local database.

    To perform the API queries it utilizes the CalendarFetcher class.


    Attributes:
        #FIXME Review attributes not needed.

    """
    queue = None
    config = None
    logger = None
    fetched_schedule_data = None
    calendar_fetcher = None
    stop_event = None


    def __init__(self, config):
        """
        Initializes the class.

        Args:
            config (AuraConfig):    The configuration
        """
        threading.Thread.__init__(self)

        self.config = config
        self.logger = logging.getLogger("AuraEngine")
        self.queue = queue.Queue()
        self.stop_event = threading.Event()
        self.calendar_fetcher = CalendarFetcher(config)



    def get_queue(self):
        """ 
        Retrieves the queue of fetched data.
        """
        return self.queue



    def run(self):
        """
        Fetch calendar data and store it in the database

        Returns
            Schedule ([]):  An arrar of retrieved schedules passed via `self.queue`
        """
        result = []

        try:
            fetched_schedule_data = self.calendar_fetcher.fetch()
            self.logger.debug("Schedule data fetched from API: " + str(fetched_schedule_data))

            # If nothing is fetched, return
            if not fetched_schedule_data:
                self.queue.put("fetching_aborted Nothing fetched")
                return
            
            # Check if existing schedules have been deleted
            local_schedules = Schedule.select_programme(datetime.now())

            for local_schedule in local_schedules:

                # Filter the local schedule from the fetched ones
                existing_schedule = list(filter(lambda new_schedule: \
                    new_schedule["schedule_id"] == local_schedule.schedule_id, fetched_schedule_data))
                
                if existing_schedule:
                    self.logger.debug("Schedule (schedule_id: %s) is still existing remotely:" % \
                        (local_schedule.schedule_id))
                else:
                    self.logger.info("Schedule (schedule_id: %s) has been deleted remotely! [%s]" % \
                        (local_schedule.schedule_id, str(local_schedule)))
                    local_schedule.delete(commit=True)
                    self.logger.info("Deleted local schedule from database (schedule_id: %s)" % local_schedule.schedule_id)


            # Process fetched schedules    
            for schedule in fetched_schedule_data:

                # Check schedule for validity
                if "start" not in schedule:
                    self.logger.warning("No 'start' of schedule given. Skipping the schedule: %s " % str(schedule))
                    continue
                if "end" not in schedule:
                    self.logger.warning("No 'end' of schedule given. Skipping the schedule: %s " % str(schedule))
                    continue

                # Store the schedule
                schedule_db = self.store_schedule(schedule)

                # Store playlists to play
                self.store_playlist(schedule_db, schedule_db.playlist_id, schedule["playlist"])
                if schedule_db.schedule_fallback_id:
                    self.store_playlist(schedule_db, schedule_db.schedule_fallback_id, schedule["schedule_fallback"], 1)
                if schedule_db.show_fallback_id:
                    self.store_playlist(schedule_db, schedule_db.show_fallback_id, schedule["show_fallback"], 2)
                if schedule_db.station_fallback_id:
                    self.store_playlist(schedule_db, schedule_db.station_fallback_id, schedule["station_fallback"], 3)



                result.append(schedule_db)

            # Release the mutex
            self.queue.put(result)
        except Exception as e:
            # Release the mutex
            self.logger.warning("Fetching aborted due to: %s" % str(e))
            self.queue.put("fetching_aborted " + str(e))

        # terminate the thread
        return



    def store_schedule(self, schedule):
        """
        Stores the given schedule to the database.

        Args:
            schedule (Schedule):    The schedule
        """
        schedule_db = Schedule.select_show_on_datetime(schedule["start"])
        havetoadd = False

        if not schedule_db:
            self.logger.debug("no schedule with given schedule id in database => create new")
            schedule_db = Schedule()
            havetoadd = True


        schedule_db.show_id = schedule["show_id"]
        schedule_db.schedule_id = schedule["schedule_id"]
        schedule_db.schedule_start = schedule["start"]
        schedule_db.schedule_end = schedule["end"]
        schedule_db.show_name = schedule["show_name"]
        schedule_db.show_hosts = schedule["show_hosts"]
        schedule_db.is_repetition = schedule["is_repetition"]
        schedule_db.funding_category = schedule["show_fundingcategory"]
        schedule_db.languages = schedule["show_languages"]
        schedule_db.type = schedule["show_type"]
        schedule_db.category = schedule["show_categories"]
        schedule_db.topic = schedule["show_topics"]
        schedule_db.musicfocus = schedule["show_musicfocus"]

        # if schedule["playlist_id"] is None:
        #     # FIXME Manually assigned playlist ID.
        #     schedule["playlist_id"] = 1

        schedule_db.playlist_id = schedule["playlist_id"]
        schedule_db.schedule_fallback_id = schedule["schedule_fallback_id"]
        schedule_db.show_fallback_id = schedule["show_fallback_id"]
        schedule_db.station_fallback_id = schedule["station_fallback_id"]

        schedule_db.store(add=havetoadd, commit=True)

        return schedule_db



    def store_playlist(self, schedule_db, playlist_id, fetched_playlist, fallbackplaylist_type=0):
        """
        Stores the Playlist to the database.
        """
        playlist_db = Playlist.select_playlist_for_schedule(schedule_db.schedule_start, playlist_id)
        havetoadd = False

        if not playlist_db:
            playlist_db = Playlist()
            havetoadd = True

        self.logger.debug("Storing playlist %d for schedule (%s)" % (playlist_id, str(schedule_db)))
        playlist_db.playlist_id = playlist_id
        playlist_db.schedule_start = schedule_db.schedule_start
        playlist_db.show_name = schedule_db.show_name
        playlist_db.fallback_type = fallbackplaylist_type
        if "entries" in fetched_playlist:
            playlist_db.entry_count = len(fetched_playlist["entries"])
        else:
            playlist_db.entry_count = 0

        playlist_db.store(havetoadd, commit=True)

        if playlist_db.entry_count > 0:
            self.store_playlist_entries(playlist_db, fetched_playlist)

        return playlist_db



    def store_playlist_entries(self, playlist_db, fetched_playlist):
        """
        Stores the playlist entries to the database.
        """
        entry_num = 0
        time_marker = playlist_db.start_unix

        for entry in fetched_playlist["entries"]:
            entry_db = PlaylistEntry.select_playlistentry_for_playlist(playlist_db.artificial_id, entry_num)
            havetoadd = False
            if not entry_db:
                entry_db = PlaylistEntry()
                havetoadd = True

            # Nano-seconds to seconds
            duration = int(float(entry["file"]["duration"]) / 1000000000) 

            entry_db.duration = duration
            entry_db.entry_start = datetime.fromtimestamp(time_marker)
            entry_db.artificial_playlist_id = playlist_db.artificial_id
            entry_db.entry_num = entry_num
            entry_db.uri = entry["uri"]
            entry_db.filename = entry["filename"]

            entry_db.store(havetoadd, commit=True)

            self.store_playlist_entry_metadata(entry_db, entry["file"]["metadata"])

            entry_num = entry_num + 1
            time_marker += duration



    def store_playlist_entry_metadata(self, entry_db, metadata):
        """
        Stores the meta-data for a PlaylistEntry.
        """
        metadata_db = PlaylistEntryMetaData.select_metadata_for_entry(entry_db.artificial_id)
        havetoadd = False
        if not metadata_db:
            metadata_db = PlaylistEntryMetaData()
            havetoadd = True

        metadata_db.artificial_entry_id = entry_db.artificial_id

        if "artist" not in metadata:
            self.logger.warning("Artist not found in metadata for track '%s'. Setting to ''" % entry_db.filename)
            metadata_db.artist = ""
        else:
            metadata_db.artist = metadata["artist"]
        
        if "album" in metadata:
            metadata_db.album = metadata["album"]

        if "artist" not in metadata:
            self.logger.warning("Title not found in metadata for track '%s'. Setting to 'n/a'" % entry_db.filename)
            metadata_db.artist = "n/a"
        else:
            metadata_db.artist = metadata["artist"]

        metadata_db.store(havetoadd, commit=True)



    def stop(self):
        self.stop_event.set()
